/**
 * @file main.c
 * @author Shinichiro Nakamura
 */

/*
 * ===============================================================
 * "Natural Tiny Basic (NT-Basic)"
 * "A tiny BASIC interpreter"
 * ---------------------------------------------------------------
 * Top of the application
 * ===============================================================
 * Copyright (c) 2012 Shinichiro Nakamura
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * ===============================================================
 */

#include <stdio.h>
#include <stdlib.h>
#include "ntbasic.h"

static char *load_program(char *filename)
{
  FILE *fp;
  long len;
  char *prog = NULL;

  fp = fopen(filename, "rb");
  if (fp == NULL) {
    return NULL;
  }

  fseek(fp, 0, SEEK_END);
  len = ftell(fp);
  fseek(fp, 0, SEEK_SET);
  prog = (char *)malloc(sizeof(char) * (len + 1));

  fread(prog, len, 1, fp);
  *(prog + len) = '\0';

  fclose(fp);

  return prog;
}

int main(int argc, char *argv[])
{
  ntbasic_t ntbasic;
  ntbasic_error_t error;
  char *program;

  /*
   * Check the arguments.
   */
  if (argc != 2) {
    printf("usage: ntbasic <filename>\n");
    return 1;
  }

  /*
   * Load program from a file system.
   */
  program = load_program(argv[1]);
  if (program == NULL) {
    printf("program read failed.\n");
    return 2;
  }

  /*
   * Setup and execute.
   */
  error = ntbasic_setup(&ntbasic, program);
  while (error == NoError) {
    error = ntbasic_execute(&ntbasic);
  }

  /*
   * Check the termination code.
   */
  if (error != EndOfTheProgram) {
    core_error_message(&ntbasic, error);
  }

  return 0;
}

